@@ -57,7 +57,9 @@
  * passphrase will be rendered as zbase32 which results for 150 bits
  * in a string of 30 characters.  That fits nicely into the 5
  * character blocking which pinentry can do.  128 bits would actually
- * be sufficient but can't be formatted nicely. */
+ * be sufficient but can't be formatted nicely.  Please do not change
+ * this value because pattern check files may let such passwords
+ * always pass. */
 #define DEFAULT_GENPIN_BITS 150
 
 /* The assuan context of the current pinentry. */
@@ -394,11 +396,7 @@ start_pinentry (ctrl_t ctrl)
 
   i=0;
   if (!opt.running_detached)
-    {
-      if (log_get_fd () != -1)
-        no_close_list[i++] = assuan_fd_from_posix_fd (log_get_fd ());
-      no_close_list[i++] = assuan_fd_from_posix_fd (fileno (stderr));
-    }
+    no_close_list[i++] = assuan_fd_from_posix_fd (fileno (stderr));
   no_close_list[i] = ASSUAN_INVALID_FD;
 
   rc = assuan_new (&ctx);
@@ -848,21 +846,20 @@ estimate_passphrase_quality (const char *pw)
 
 
 /* Generate a random passphrase in zBase32 encoding (RFC-6189) to be
- * used by Pinentry to suggest a passphrase.  */
+ * used by Pinentry to suggest a passphrase.  Note that we have the
+ * same algorithm in gpg.c for --gen-random at level 30.  It is
+ * important that we always output exactly 30 characters to match the
+ * special exception we have in the pattern file for symmetric
+ * encryption.  */
 static char *
 generate_pin (void)
 {
-  unsigned int nbits = opt.min_passphrase_len * 8;
-  size_t nbytes;
+  unsigned int nbits = DEFAULT_GENPIN_BITS;
+  size_t nbytes = nbytes = (nbits + 7) / 8;
   void *rand;
   char *generated;
 
-  if (nbits < 128)
-    nbits = DEFAULT_GENPIN_BITS;
-
-  nbytes = (nbits + 7) / 8;
-
-  rand = gcry_random_bytes_secure (nbytes, GCRY_STRONG_RANDOM);
+   rand = gcry_random_bytes_secure (nbytes, GCRY_STRONG_RANDOM);
   if (!rand)
     {
       log_error ("failed to generate random pin\n");
@@ -1065,10 +1062,10 @@ static gpg_error_t
 setup_formatted_passphrase (ctrl_t ctrl)
 {
   static const struct { const char *key, *help_id, *value; } tbl[] = {
-    /* TRANSLATORS: This is a text shown by pinentry if the option
-        for formatted passphrase is enabled.  The length is
-        limited to about 900 characters.  */
     { "hint",  "pinentry.formatted_passphrase.hint",
+      /* TRANSLATORS: This is a text shown by pinentry if the option
+         for formatted passphrase is enabled.  The length is
+         limited to about 900 characters.  */
       N_("Note: The blanks are not part of the passphrase.") },
     { NULL, NULL }
   };
@@ -1546,6 +1543,17 @@ agent_askpin (ctrl_t ctrl,
                             NULL, NULL, NULL, NULL, NULL, NULL);
       if (rc)
         pininfo->with_repeat = 0; /* Pinentry does not support it.  */
+
+      if (pininfo->with_repeat)
+        {
+          snprintf (line, DIM(line), "SETREPEATOK %s",
+                    L_("Passphrases match."));
+          rc = assuan_transact (entry_ctx, line,
+                                NULL, NULL, NULL, NULL, NULL, NULL);
+          if (rc)
+            rc = 0; /* Pinentry does not support it. */
+        }
+
     }
   pininfo->repeat_okay = 0;
   pininfo->status = 0;
@@ -1805,6 +1813,16 @@ agent_get_passphrase (ctrl_t ctrl,
       if (rc)
         pininfo->with_repeat = 0; /* Pinentry does not support it.  */
 
+      if (pininfo->with_repeat)
+        {
+          snprintf (line, DIM(line), "SETREPEATOK %s",
+                    L_("Passphrases match."));
+          rc = assuan_transact (entry_ctx, line,
+                                NULL, NULL, NULL, NULL, NULL, NULL);
+          if (rc)
+            rc = 0; /* Pinentry does not support it. */
+        }
+
       (void)setup_genpin (ctrl);
 
       rc = setup_enforced_constraints (ctrl);
