@@ -314,7 +314,7 @@ agent_pksign_do (ctrl_t ctrl, const char *cache_nonce,
   if (!ctrl->have_keygrip)
     return gpg_error (GPG_ERR_NO_SECKEY);
 
-  err = agent_key_from_file (ctrl, cache_nonce, desc_text, ctrl->keygrip,
+  err = agent_key_from_file (ctrl, cache_nonce, desc_text, NULL,
                              &shadow_info, cache_mode, lookup_ttl,
                              &s_skey, NULL, NULL);
   if (gpg_err_code (err) == GPG_ERR_NO_SECKEY)
@@ -372,8 +372,15 @@ agent_pksign_do (ctrl_t ctrl, const char *cache_nonce,
             }
 
           if (keyref)
-            agent_write_shadow_key (ctrl->keygrip, serialno, keyref, pkbuf, 0);
+            {
+              char *dispserialno;
 
+              agent_card_getattr (ctrl, "$DISPSERIALNO", &dispserialno,
+                                  hexgrip);
+              agent_write_shadow_key (ctrl->keygrip, serialno, keyref, pkbuf,
+                                      0, dispserialno);
+              xfree (dispserialno);
+            }
           algo = get_pk_algo_from_key (s_pkey);
 
           xfree (serialno);
@@ -392,23 +399,17 @@ agent_pksign_do (ctrl_t ctrl, const char *cache_nonce,
         }
 
       {
-        char *desc2 = NULL;
-
-        if (desc_text)
-          agent_modify_description (desc_text, NULL, s_pkey, &desc2);
-
 	if (agent_is_tpm2_key (s_skey))
-	  err = divert_tpm2_pksign (ctrl, desc2? desc2 : desc_text,
+	  err = divert_tpm2_pksign (ctrl,
 				    data, datalen,
 				    ctrl->digest.algo,
 				    shadow_info, &buf, &len);
 	else
-	  err = divert_pksign (ctrl, desc2? desc2 : desc_text,
+	  err = divert_pksign (ctrl,
 			       ctrl->keygrip,
 			       data, datalen,
 			       ctrl->digest.algo,
-			       shadow_info, &buf, &len);
-        xfree (desc2);
+			       &buf, &len);
       }
       if (err)
         {
