@@ -609,7 +609,7 @@ gnupg_tmpfile (void)
   char *name, *p;
   HANDLE file;
   int pid = GetCurrentProcessId ();
-  unsigned int value;
+  unsigned int value = 0;
   int i;
   SECURITY_ATTRIBUTES sec_attr;
 
@@ -634,12 +634,9 @@ gnupg_tmpfile (void)
   for (attempts=0; attempts < 10; attempts++)
     {
       p = name;
-      value = (GetTickCount () ^ ((pid<<16) & 0xffff0000));
+      value += (GetTickCount () ^ ((pid<<16) & 0xffff0000));
       for (i=0; i < 8; i++)
-        {
-          *p++ = tohex (((value >> 28) & 0x0f));
-          value <<= 4;
-        }
+	*p++ = tohex (((value >> (7 - i)*4) & 0x0f));
       strcpy (p, ".tmp");
       file = CreateFile (buffer,
                          GENERIC_READ | GENERIC_WRITE,
@@ -778,8 +775,12 @@ gnupg_allow_set_foregound_window (pid_t pid)
   else if (inhibit_set_foregound_window)
     ;
   else if (!AllowSetForegroundWindow ((pid_t)pid == (pid_t)(-1)?ASFW_ANY:pid))
-    log_info ("AllowSetForegroundWindow(%lu) failed: %s\n",
-               (unsigned long)pid, w32_strerror (-1));
+    {
+      char *flags = getenv ("GNUPG_EXEC_DEBUG_FLAGS");
+      if (flags && (atoi (flags) & 2))
+        log_info ("AllowSetForegroundWindow(%lu) failed: %s\n",
+                  (unsigned long)pid, w32_strerror (-1));
+    }
 #endif
 }
 
@@ -1191,19 +1192,28 @@ gnupg_unsetenv (const char *name)
 #else /*!HAVE_UNSETENV*/
   {
     char *buf;
+    int r;
 
     if (!name)
       {
         gpg_err_set_errno (EINVAL);
         return -1;
       }
-    buf = xtrystrdup (name);
+    buf = strconcat (name, "=", NULL);
     if (!buf)
       return -1;
+
+    r = putenv (buf);
+# ifdef HAVE_W32_SYSTEM
+    /* For Microsoft implementation, we can free the memory in this
+       use case.  */
+    xfree (buf);
+# else
 #  if __GNUC__
 #   warning no unsetenv - trying putenv but leaking memory.
 #  endif
-    return putenv (buf);
+# endif
+    return r;
   }
 #endif /*!HAVE_UNSETENV*/
 }
@@ -1233,10 +1243,20 @@ int
 gnupg_stat (const char *name, struct stat *statbuf)
 {
 # ifdef HAVE_W32_SYSTEM
+#  if __MINGW32_MAJOR_VERSION > 3
+    /* mingw.org's MinGW */
+#   define STRUCT_STAT _stat
+#  elif defined(_USE_32BIT_TIME_T)
+    /* MinGW64 for i686 */
+#   define STRUCT_STAT _stat32
+#  else
+    /* MinGW64 for x86_64 */
+#   define STRUCT_STAT _stat64i32
+#  endif
   if (any8bitchar (name))
     {
       wchar_t *wname;
-      struct _stat32 st32;
+      struct STRUCT_STAT st32;
       int ret;
 
       wname = utf8_to_wchar (name);
