@@ -32,6 +32,7 @@ struct agent_card_info_s
   int error;         /* private. */
   char *reader;      /* Reader information.  */
   char *apptype;     /* Malloced application type string.  */
+  unsigned int appversion; /* Version of the application.  */
   unsigned int manufacturer_id;
   char *manufacturer_name; /* malloced.  */
   char *serialno;    /* malloced hex string. */
@@ -107,6 +108,9 @@ gpg_error_t agent_scd_keypairinfo (ctrl_t ctrl, const char *keyref,
 /* Return list of cards.  */
 int agent_scd_cardlist (strlist_t *result);
 
+/* Switch/assure a certain application. */
+gpg_error_t agent_scd_switchapp (const char *appname);
+
 /* Free a keypair info list.  */
 void free_keypair_info (keypair_info_t l);
 
@@ -131,7 +135,8 @@ int agent_keytotpm (ctrl_t ctrl, const char *hexgrip);
 
 /* Send the KEYTOCARD command. */
 int agent_keytocard (const char *hexgrip, int keyno, int force,
-                     const char *serialno, const char *timestamp);
+                     const char *serialno, const char *timestamp,
+                     const char *ecdh_param_str);
 
 /* Send a SETATTR command to the SCdaemon. */
 gpg_error_t agent_scd_setattr (const char *name,
@@ -230,7 +235,7 @@ gpg_error_t agent_import_key (ctrl_t ctrl, const char *desc,
 /* Receive a key from the agent.  */
 gpg_error_t agent_export_key (ctrl_t ctrl, const char *keygrip,
                               const char *desc, int openpgp_protected,
-                              char **cache_nonce_addr,
+                              int mode1003, char **cache_nonce_addr,
                               unsigned char **r_result, size_t *r_resultlen,
                               u32 *keyid, u32 *mainkeyid, int pubkey_algo);
 
