@@ -64,6 +64,8 @@ struct isvalid_status_parm_s {
   ctrl_t ctrl;
   int seen;
   unsigned char fpr[20];
+  gnupg_isotime_t revoked_at;
+  char *revocation_reason;  /* malloced or NULL */
 };
 
 
@@ -360,7 +362,7 @@ inq_certificate (void *opaque, const char *line)
     }
   else
     {
-      log_error ("unsupported inquiry '%s'\n", line);
+      log_error ("unsupported certificate inquiry '%s'\n", line);
       return gpg_error (GPG_ERR_ASS_UNKNOWN_INQUIRE);
     }
 
@@ -384,8 +386,8 @@ inq_certificate (void *opaque, const char *line)
       int err;
       ksba_cert_t cert;
 
-
-      err = gpgsm_find_cert (parm->ctrl, line, ski, &cert, 1);
+      err = gpgsm_find_cert (parm->ctrl, line, ski, &cert,
+                             FIND_CERT_ALLOW_AMBIG|FIND_CERT_WITH_EPHEM);
       if (err)
         {
           log_error ("certificate not found: %s\n", gpg_strerror (err));
@@ -425,6 +427,51 @@ unhexify_fpr (const char *hexstr, unsigned char *fpr)
 }
 
 
+/* This is a helper to print diagnostics from dirmngr indicated by
+ * WARNING or NOTE status lines.  Returns true if the status LINE was
+ * processed.  */
+static int
+warning_and_note_printer (const char *line)
+{
+  const char *s, *s2;
+  const char *warn = NULL;
+  int is_note = 0;
+
+  if ((s = has_leading_keyword (line, "WARNING")))
+    ;
+  else if ((is_note = !!(s = has_leading_keyword (line, "NOTE"))))
+    ;
+  else
+    return 0;  /* Nothing to process.  */
+
+  if ((s2 = has_leading_keyword (s, "no_crl_due_to_tor"))
+      || (s2 = has_leading_keyword (s, "no_ldap_due_to_tor"))
+      || (s2 = has_leading_keyword (s, "no_ocsp_due_to_tor")))
+    warn = _("Tor might be in use - network access is limited");
+  else
+    warn = NULL;
+
+  if (warn)
+    {
+      if (is_note)
+        log_info (_("Note: %s\n"), warn);
+      else
+        log_info (_("WARNING: %s\n"), warn);
+      if (s2)
+        {
+          while (*s2 && !spacep (s2))
+            s2++;
+          while (*s2 && spacep (s2))
+            s2++;
+          if (*s2)
+            gpgsm_print_further_info ("%s", s2);
+        }
+    }
+
+  return 1;  /* Status line processed.  */
+}
+
+
 static gpg_error_t
 isvalid_status_cb (void *opaque, const char *line)
 {
@@ -446,6 +493,23 @@ isvalid_status_cb (void *opaque, const char *line)
       if (!*s || !unhexify_fpr (s, parm->fpr))
         parm->seen++; /* Bump it to indicate an error. */
     }
+  else if ((s = has_leading_keyword (line, "REVOCATIONINFO")))
+    {
+      if (*s && strlen (s) >= 15)
+        {
+          memcpy (parm->revoked_at, s, 15);
+          parm->revoked_at[15] = 0;
+        }
+      s += 15;
+      while (*s && spacep (s))
+        s++;
+      xfree (parm->revocation_reason);
+      parm->revocation_reason = *s? xtrystrdup (s) : NULL;
+    }
+  else if (warning_and_note_printer (line))
+    {
+    }
+
   return 0;
 }
 
@@ -457,16 +521,22 @@ isvalid_status_cb (void *opaque, const char *line)
 
   GPG_ERR_CERTIFICATE_REVOKED
   GPG_ERR_NO_CRL_KNOWN
+  GPG_ERR_INV_CRL_OBJ
   GPG_ERR_CRL_TOO_OLD
 
   Values for USE_OCSP:
      0 = Do CRL check.
-     1 = Do an OCSP check but fallback to CRL unless CRLS are disabled.
-     2 = Do only an OCSP check using only the default responder.
+     1 = Do an OCSP check but fallback to CRL unless CRLs are disabled.
+     2 = Do only an OCSP check (used for the chain model).
+
+   If R_REVOKED_AT pr R_REASON are not NULL and the certificate has
+   been revoked the revocation time and the reason are copied to there.
+   The caller needs to free R_REASON.
  */
-int
+gpg_error_t
 gpgsm_dirmngr_isvalid (ctrl_t ctrl,
-                       ksba_cert_t cert, ksba_cert_t issuer_cert, int use_ocsp)
+                       ksba_cert_t cert, ksba_cert_t issuer_cert, int use_ocsp,
+                       gnupg_isotime_t r_revoked_at, char **r_reason)
 {
   static int did_options;
   int rc;
@@ -475,6 +545,11 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
   struct inq_certificate_parm_s parm;
   struct isvalid_status_parm_s stparm;
 
+  if (r_revoked_at)
+    *r_revoked_at = 0;
+  if (r_reason)
+    *r_reason = NULL;
+
   rc = start_dirmngr (ctrl);
   if (rc)
     return rc;
@@ -504,6 +579,8 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
   stparm.ctrl = ctrl;
   stparm.seen = 0;
   memset (stparm.fpr, 0, 20);
+  stparm.revoked_at[0] = 0;
+  stparm.revocation_reason = NULL;
 
   /* It is sufficient to send the options only once because we have
    * one connection per process only.  */
@@ -514,9 +591,8 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
                          NULL, NULL, NULL, NULL, NULL, NULL);
       did_options = 1;
     }
-  snprintf (line, DIM(line), "ISVALID%s%s %s%s%s",
-            use_ocsp == 2 || opt.no_crl_check ? " --only-ocsp":"",
-            use_ocsp == 2? " --force-default-responder":"",
+  snprintf (line, DIM(line), "ISVALID%s %s%s%s",
+            (use_ocsp == 2 || opt.no_crl_check) ? " --only-ocsp":"",
             certid,
             use_ocsp? " ":"",
             use_ocsp? certfpr:"");
@@ -529,6 +605,19 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
   if (opt.verbose > 1)
     log_info ("response of dirmngr: %s\n", rc? gpg_strerror (rc): "okay");
 
+  if (gpg_err_code (rc) == GPG_ERR_CERT_REVOKED
+      && !check_isotime (stparm.revoked_at))
+    {
+      if (r_revoked_at)
+        gnupg_copy_time (r_revoked_at, stparm.revoked_at);
+      if (r_reason)
+        {
+          *r_reason = stparm.revocation_reason;
+          stparm.revocation_reason = NULL;
+        }
+
+    }
+
   if (!rc && stparm.seen)
     {
       /* Need to also check the certificate validity. */
@@ -572,7 +661,8 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
                 {
                   /* Note the no_dirmngr flag: This avoids checking
                      this certificate over and over again. */
-                  rc = gpgsm_validate_chain (ctrl, rspcert, "", NULL, 0, NULL,
+                  rc = gpgsm_validate_chain (ctrl, rspcert, GNUPG_ISOTIME_NONE,
+                                             NULL, 0, NULL,
                                              VALIDATE_FLAG_NO_DIRMNGR, NULL);
                   if (rc)
                     {
@@ -585,7 +675,9 @@ gpgsm_dirmngr_isvalid (ctrl_t ctrl,
           ksba_cert_release (rspcert);
         }
     }
+
   release_dirmngr (ctrl);
+  xfree (stparm.revocation_reason);
   return rc;
 }
 
@@ -721,6 +813,10 @@ lookup_status_cb (void *opaque, const char *line)
           gpgsm_status (parm->ctrl, STATUS_TRUNCATED, line);
         }
     }
+  else if (warning_and_note_printer (line))
+    {
+    }
+
   return 0;
 }
 
@@ -919,7 +1015,8 @@ run_command_inq_cb (void *opaque, const char *line)
       if (!*line)
         return gpg_error (GPG_ERR_ASS_PARAMETER);
 
-      err = gpgsm_find_cert (parm->ctrl, line, NULL, &cert, 1);
+      err = gpgsm_find_cert (parm->ctrl, line, NULL, &cert,
+                             FIND_CERT_ALLOW_AMBIG);
       if (err)
         {
           log_error ("certificate not found: %s\n", gpg_strerror (err));
@@ -940,9 +1037,33 @@ run_command_inq_cb (void *opaque, const char *line)
       line = s;
       log_info ("dirmngr: %s\n", line);
     }
+  else if ((s = has_leading_keyword (line, "ISTRUSTED")))
+    {
+      /* The server is asking us whether the certificate is a trusted
+         root certificate.  */
+      char fpr[41];
+      struct rootca_flags_s rootca_flags;
+      int n;
+
+      line = s;
+
+      for (s=line,n=0; hexdigitp (s); s++, n++)
+        ;
+      if (*s || n != 40)
+        return gpg_error (GPG_ERR_ASS_PARAMETER);
+      for (s=line, n=0; n < 40; s++, n++)
+        fpr[n] = (*s >= 'a')? (*s & 0xdf): *s;
+      fpr[n] = 0;
+
+      if (!gpgsm_agent_istrusted (parm->ctrl, NULL, fpr, &rootca_flags))
+        rc = assuan_send_data (parm->ctx, "1", 1);
+      else
+        rc = 0;
+      return rc;
+    }
   else
     {
-      log_error ("unsupported inquiry '%s'\n", line);
+      log_error ("unsupported command inquiry '%s'\n", line);
       rc = gpg_error (GPG_ERR_ASS_UNKNOWN_INQUIRE);
     }
 
@@ -968,6 +1089,10 @@ run_command_status_cb (void *opaque, const char *line)
             return gpg_error (GPG_ERR_ASS_CANCELED);
         }
     }
+  else if (warning_and_note_printer (line))
+    {
+    }
+
   return 0;
 }
 
