@@ -134,9 +134,6 @@ close_message_fd (ctrl_t ctrl)
 {
   if (ctrl->server_local->message_fd != -1)
     {
-#ifdef HAVE_W32CE_SYSTEM
-#warning Is this correct for W32/W32CE?
-#endif
       close (ctrl->server_local->message_fd);
       ctrl->server_local->message_fd = -1;
     }
@@ -724,8 +721,13 @@ cmd_export (assuan_context_t ctx, char *line)
 
   if (opt_secret)
     {
-      if (!list || !*list->d)
+      if (!list)
         return set_error (GPG_ERR_NO_DATA, "No key given");
+      if (!*list->d)
+        {
+          free_strlist (list);
+          return set_error (GPG_ERR_NO_DATA, "No key given");
+        }
       if (list->next)
         return set_error (GPG_ERR_TOO_MANY, "Only one key allowed");
   }
@@ -866,12 +868,6 @@ cmd_message (assuan_context_t ctx, char *line)
   if (rc)
     return rc;
 
-#ifdef HAVE_W32CE_SYSTEM
-  sysfd = _assuan_w32ce_finish_pipe ((int)sysfd, 0);
-  if (sysfd == INVALID_HANDLE_VALUE)
-    return set_error (gpg_err_code_from_syserror (),
-		      "rvid conversion failed");
-#endif
 
   fd = translate_sys2libc_fd (sysfd, 0);
   if (fd == -1)
@@ -1014,17 +1010,27 @@ do_listkeys (assuan_context_t ctx, char *line, int mode)
       int outfd = translate_sys2libc_fd (assuan_get_output_fd (ctx), 1);
 
       if ( outfd == -1 )
-        return set_error (GPG_ERR_ASS_NO_OUTPUT, NULL);
+        {
+          free_strlist (list);
+          return set_error (GPG_ERR_ASS_NO_OUTPUT, NULL);
+        }
       fp = es_fdopen_nc (outfd, "w");
       if (!fp)
-        return set_error (gpg_err_code_from_syserror (), "es_fdopen() failed");
+        {
+          free_strlist (list);
+          return set_error (gpg_err_code_from_syserror (),
+                            "es_fdopen() failed");
+        }
     }
   else
     {
       fp = es_fopencookie (ctx, "w", data_line_cookie_functions);
       if (!fp)
-        return set_error (GPG_ERR_ASS_GENERAL,
-                          "error setting up a data stream");
+        {
+          free_strlist (list);
+          return set_error (GPG_ERR_ASS_GENERAL,
+                            "error setting up a data stream");
+        }
     }
 
   ctrl->with_colons = 1;
@@ -1034,6 +1040,7 @@ do_listkeys (assuan_context_t ctx, char *line, int mode)
   if (ctrl->server_local->list_external)
     listmode |= (1<<7);
   err = gpgsm_list_keys (assuan_get_pointer (ctx), list, fp, listmode);
+
   free_strlist (list);
   es_fclose (fp);
   if (ctrl->server_local->list_to_output)
@@ -1354,13 +1361,9 @@ gpgsm_server (certlist_t default_recplist)
   /* We use a pipe based server so that we can work from scripts.
      assuan_init_pipe_server will automagically detect when we are
      called with a socketpair and ignore FILEDES in this case. */
-#ifdef HAVE_W32CE_SYSTEM
-  #define SERVER_STDIN es_fileno(es_stdin)
-  #define SERVER_STDOUT es_fileno(es_stdout)
-#else
 #define SERVER_STDIN 0
 #define SERVER_STDOUT 1
-#endif
+
   filedes[0] = assuan_fdopen (SERVER_STDIN);
   filedes[1] = assuan_fdopen (SERVER_STDOUT);
   rc = assuan_new (&ctx);
@@ -1527,7 +1530,7 @@ gpgsm_status_with_err_code (ctrl_t ctrl, int no, const char *text,
 {
   char buf[30];
 
-  sprintf (buf, "%u", (unsigned int)ec);
+  snprintf (buf, sizeof buf, "%u", (unsigned int)ec);
   if (text)
     return gpgsm_status2 (ctrl, no, text, buf, NULL);
   else
